########################################################################
# 2025 April 14
#
# The author disclaims copyright to this source code.  In place of
# a legal notice, here is a blessing:
#
#  * May you do good and not evil.
#  * May you find forgiveness for yourself and forgive others.
#  * May you share freely, never taking more than you give.
#
########################################################################
# ----- @module teaish-checks.tcl -----
# @section TEA-ish collection of feature tests.
#
# Functions for feature-checking for OpenSSL in teaish.

# @teaish-check-pkg-config-openssl
#
# Returns 1 if openssl is found via pkg-config, else 0.  If found,
# passes its link flags to teaish-ldflags-prepend.
#
# It defines LDFLAGS_OPENSSL to the linker flags and CFLAGS_OPENSSL to
# the CFLAGS, or "" if it's not found.
#
# Defines HAVE_OPENSSL to 0 or 1 (its return value).
#
# If it returns true, the underlying pkg-config test will set several
# defines named PKG_OPENSSL_... (see the docs for [pkg-config] for
# details).
proc teaish-check-pkg-config-openssl {} {
  use pkg-config
  teaish-check-cached -nostatus "Checking for openssl via pkg-config" {
    set rc 0
    if {[msg-quiet pkg-config-init 0] && [msg-quiet pkg-config openssl]} {
      set lfl [string trim "[get-define PKG_OPENSSL_LDFLAGS] [get-define PKG_OPENSSL_LIBS]"]
      define CFLAGS_OPENSSL [get-define PKG_OPENSSL_CFLAGS]
      define LDFLAGS_OPENSSL $lfl
      teaish-ldflags-prepend $lfl
      msg-result "ok ($lfl)"
      incr rc
    } else {
      define CFLAGS_OPENSSL ""
      define LDFLAGS_OPENSSL ""
      msg-result "no"
    }
    define HAVE_OPENSSL $rc
    expr $rc
  }
}

# Internal helper for OpenSSL checking using cc-with to check if the
# given $cflags, $ldflags, and list of -l libs can link an
# application.
#
# For a system-level check, use empty $cflags and $ldflags.
#
# On success, it defines CFLAGS_OPENSSL to $cflags and LDFLAGS_OPENSSL
# to a combination of $ldflags and any required libs (which may be
# amended beyond those provided in $libs). It then returns 1.
#
# On failure it defines the above-mentioned flags to ""
# and returns 0.
#
# Defines HAVE_OPENSSL to its return value.
#
# Derived from https://fossil-scm.org/file/auto.def
proc teaish__check-openssl {msg cflags ldflags {libs {-lssl -lcrypto -lpthread}}} {
  msg-checking "$msg ... "
  set rc 0
  set isMinGw [teaish-is-mingw]
  if {$isMinGw} {
    lappend libs -lgdi32 -lwsock32 -lcrypt32
  }
  set prefix msg-quiet
  #set prefix ""
  set lz ""
  if {[{*}$prefix teaish-check-libz]} {
    set lz [get-define LDFLAGS_LIBZ]
  }
  set libs2 $libs
  if {$lz ne ""} {
    lappend libs2 $lz
  }
  {*}$prefix cc-with [list -link 1 -cflags "$cflags $ldflags" -libs $libs2] {
    if {[cc-check-includes openssl/ssl.h] && \
          [cc-check-functions SSL_new]} {
      incr rc
    }
  }
  if {!$rc && !$isMinGw} {
    # On some systems, OpenSSL appears to require -ldl to link.
    if {[{*}$prefix teaish-check-dlopen]} {
      lappend libs2 [get-define LDFLAGS_DLOPEN ""]
      {*}$prefix cc-with [list  -link 1 -cflags "$cflags $ldflags" -libs $libs2] {
        if {[cc-check-includes openssl/ssl.h] && \
              [cc-check-functions SSL_new]} {
          incr rc
        }
      }
    }
  }
  #puts "*???? cflags=$cflags ldflags=$ldflags libs2=$libs2"
  if {$rc} {
    msg-result "ok"
    define CFLAGS_OPENSSL "$cflags"
    define LDFLAGS_OPENSSL "$ldflags $libs2"
  } else {
    define CFLAGS_OPENSSL ""
    define LDFLAGS_OPENSSL ""
    msg-result "no"
  }
  define HAVE_OPENSSL $rc
  return $rc
}

# @teaish-check-openssl
#
# Jumps through some proverbial hoops to look for OpenSSL dev pieces.
#
# $where must be one of the following:
#
# - "pkg-config": check only pkg-config for it, but also verify that
#   the results from pkg-config seem to work.
#
# - "system": look in standard(ish) system paths, starting with
#   a lookup requiring no -L flag.
#
# - "auto" or "": try (pkg-config, system), in that order.
#
# - "none": do no lookup, define vars (see below), and return 0.
#
# - Any other value is assumed to be a directory name suitable for
#   finding OpenSSL, but how this lookup is run is not currently
#   well-defined.
#
# It defines LDFLAGS_OPENSSL and CFLAGS_OPENSSL to ldflags
# resp. cflags needed for compiling and linking, and updates teaish's
# internal ldflags/cflags lists. If OpenSSL is not found, they're
# defined to "".
#
# It defines HAVE_OPENSSL to 0 or 1 (its return value).
#
# If $where is empty then it defaults to auto. If $where is not empty
# _and_ OpenSSL is not found
#
# If the --with-openssl=... config flag is defined (see
# teaish-check-openssl-options) then an empty $where value will use
# the value of the --with-openssl flag, defaulting to "auto" if that
# flag also has an empty value. If that flag is provided, and has a
# value other than "none", then failure to find the library is
# considered fatal.
#
# Derived from https://fossil-scm.org/file/auto.def
proc teaish-check-openssl {{where ""}} {
  teaish-check-cached -nostatus "Looking for openssl" {
    if {$where eq ""} {
      if {[proj-opt-exists with-openssl]} {
        set where [join [opt-val with-openssl auto]]
      }
    }

    set notfound {{checkWithFlag msg} {
      if {$checkWithFlag && [proj-opt-was-provided with-openssl]} {
        proj-fatal "--with-openssl " \
          "found no working installation. Try --with-openssl=none"
      }
      define LDFLAGS_OPENSSL ""
      define CFLAGS_OPENSSL ""
      define HAVE_OPENSSL 0
      msg-result $msg
    }}

    switch -exact -- $where {
      none {
        apply $notfound 0 none
        return 0
      }
      "" {
        set where auto
      }
    }
    if {$where in {pkg-config auto}} {
      # Check pkg-config
      set x [teaish-check-pkg-config-openssl]
      #puts "***** x = <<$x>>"
      if {$x} {
        set cflags [get-define PKG_OPENSSL_CFLAGS ""]
        set ldflags [get-define LDFLAGS_OPENSSL ""]
        if {[teaish__check-openssl "Verifying openssl pkg-config values" \
               $cflags $ldflags]} {
          return 1
        }
      }
      if {$where eq "pkg-config"} {
        apply $notfound 1 "not found"
        return 0
      }
    }

    # Determine which dirs to search...
    set ssldirs {}
    if {$where in {auto system}} {
      set ssldirs {
        {} /usr/sfw /usr/local/ssl /usr/lib/ssl /usr/ssl
        /usr/pkg /usr/local /usr /usr/local/opt/openssl
        /opt/homebrew/opt/openssl
      }
    } elseif {$where ne ""} {
      lappend ssldirs $where
    }

    foreach dir $ssldirs {
      set msg "in $dir"
      set cflags "-I$dir/include"
      if {$dir eq ""} {
        set msg "without -L/path"
        set ldflags ""
        set cflags ""
      } elseif {![file isdirectory $dir]} {
        continue
      } elseif {[file readable $dir/libssl.a]} {
        set ldflags -L$dir
      } elseif {[file readable $dir/lib/libssl.a]} {
        set ldflags -L$dir/lib
      } elseif {[file isdirectory $dir/lib]} {
        set ldflags [list -L$dir -L$dir/lib]
      } else {
        set ldflags -L$dir
      }
      if {[teaish__check-openssl $msg $cflags $ldflags]} {
        teaish-cflags-add [get-define CFLAGS_OPENSSL]
        teaish-ldflags-prepend [get-define LDFLAGS_OPENSSL]
        return 1
      }
      if {$dir ne ""} {
        # Look for a static build under $dir...
        set ldflags ""
        set libs [list $dir/libssl.a $dir/libcrypto.a]
        set foundLibs 0
        foreach x $libs {
          if {![file readlable $x]} break;
          incr foundLibs
        }
        if {$foundLibs != [llength $libs]} {
          continue
        }
        set cflags "-I$dir/include"
        lappend libs -lpthread
        if {[teaish__check-openssl "Checking for static openssl build in $dir" \
               $cflags $ldflags $libs]} {
          teaish-cflags-add [get-define CFLAGS_OPENSSL]
          teaish-ldflags-prepend [get-define LDFLAGS_OPENSSL]
          return 1
        }
      }
    }

    apply $notfound 1 no
    return 0
  }
}; # teaish-check-openssl

# @teaish-check-openssl-options
#
# teaish.tcl files which use teaish-check-openssl should
# include this function's result from their teaish-options
# impl, so that configure --help can include the --with-openssl
# flag that check exposes.
#
# Returns a list of options for the teaish-check-openssl feature test.
#
# Example usage:
#
# proc teaish-options {} {
#  use teaish-feature-tests
#  return [teaish-combine-option-lists \
#            [teaish-check-openssl-options] \
#            { hell-world => {just testing} } \
#          ]
# }
proc teaish-check-openssl-options {} {
  return {
    with-openssl:see-the-help =>
      {Checks for OpenSSL development libraries in a variety of ways.
      "pkg-config" only checks the system's pkg-config.
      "system" checks only for a system-level copy.
      "auto" checks the prior options and a list of
      likely candidate locations. "none" disables the check
      altogether and causes the check to not fail if it's
      not found. Any other value is a directory in which a
      _static_ copy of libssl.a can be found, e.g. a locally-built
      copy of the OpenSSL source tree. If this flag is explicitly provided,
      and has a value other than "none", failure to find OpenSSL
      is fatal.}
  }
}
