Authoring & Building Tcl Extensions with Teaish
========================================================================

<div class='hidden'>If you are reading this from a local copy, as
opposed to from a Fossil web interface, many of the content links will
not work, as they refer to in-repository content.</div>

- [Creating a New Extension](#create)
- [Invoking Extension Builds](#build)
- [(Un)Installing Extension](#install)
- [Extension Definition Files](#files)
  - [Core extension definition (**`teaish.tcl`**)](#tcl)
  - [Building (**`teaish.make.in`**)](#make)
     - [Hook into various build phases](#hooks)
  - [Add custom Tcl init code (**`teaish.pkginit.tcl`**)](#pkginit)
  - [Customize package loading (**`pkgIndex.tcl.in`**)](#pkgindex)
  - [Test (**`teaish.test.tcl`**)](#test)
- [Distributing Extensions](#dist)
- [Template File Filtering](#filtering)

Intro
========================================================================

This page covers the build-related aspects of Tcl extensions, as they
pertain to teaish. It does not cover the C-related details of creating
Tcl extensions.  For example purposes this document uses the SQLite
Tcl extension because it's readily handy and was the initial
motiviation for this project.

Having at least a basic working knowledge of [Autosetup][] will
be helpful for using teaish.

Creating a teaish build requires some combination of the following
files:

- [`teaish.tcl`](#tcl) (required, though it may be empty for
  extremely basic cases)\  
  This is loaded by the teaish [Autosetup][] module to incorporate an
  extension's customizations, e.g. `--configure-flags`, into the
  build.

- [`teaish.make.in`](#make) (optional) is a makefile stub which
  gets `include`d by the teaish-generated Makefile. It needs to define
  certain variables so that the extension-agnostic part of the build
  can know how to build the extension.

- [`teaish.test.tcl.in` or `teaish.test.tcl.in`](#test) (optional)\  
  Can be used to create tests for the extension, such that `make test`
  will "just work."

- [`teaish.pkginit.tcl.in` or `teaish.pkginit.tcl`](#pkginit) (optional)\  
  An optional script of code to be run immediately after the module is
  loaded using either [the `make test` rules](#test) or the default
  `pkgIndex.tcl`.

- [`pkgIndex.tcl.in` or `pkgIndex.tcl`](#pkgindex) (optional)\  
  A custom `pkgIndex.tcl` for the package. The default is suitable
  for most cases.

These are covered in more detail in their own sections below, but see
the next section for how to quickly generate new ones.

Example builds:

- [**Hello, World**](/dir/teaish/example/hello) is a hello-world
  extension taken from the Tcl wiki.
- [**SQLite3**](/dir/teaish/example/sqlite) with many
  extension-specific configuration options.
- [**Also SQLite3**](/dir/teaish/example/sqlite-alt) with most config choices
  made for the user (based off of historical SQLite TEA configurations).
  This one doesn't require its own makefile.

<a id="create"></a>
Creating a New Extension
========================================================================

The easiest way to create a new set of teaish build files is:

>
```
$ /path/to/teaish/configure \
  --teaish-create-extension=/output/dir
```

Which will create the output dir (if needed) and write the following files to it:

- [**`teaish.tcl`**](#tcl)
- [**`teaish.make.in`**](#make)
- [**`teaish.test.tcl`**](#test)
- **`teaish.c`**: a hello-world-style extension

If all goes well then it should build as-is with:

>
```
$ cd the/output/dir
$ /path/to/teaish/configure \
  [--with-tcl=/path/to/tcl/install/prefix] # if needed
$ make test
```

And [(un)installed](#install) with:

>
```
$ make install   # will run an additional test
$ make uninstall # in case of later regret!
```

It can be [packaged up](#dist) for distribution with:

>
```
$ make dist
```

<a id="build"></a>
Invoking Extension Builds
========================================================================

There are three ways to plug an extension into this build system:

1. Place a copy of the the extension's teaish files in the main teaish
   directory and run `./configure`.

2. From a directory containing the extension's teaish files
   run `/path/to/tea/configure`.

3. From a completely unrelated directory run\  
   `/path/to/teaish/configure --teaish-extension-dir=/path/to/extension ...`

(Pass `--help` to the `configure` to see the list of options.)

After that, just type `make test` and (if all goes well)...

<a id="install"></a>
(Un)Installing Extensions
========================================================================

Simply:

>
```
$ make install
```

That will build the extension, if needed, but it won't implicitly run
`make test`.

Immediately after the installation, the makefile will run a "package
require" test on the extension to ensure that the newly-installed
extension can be loaded that way. It will complain loudly if that
part fails.

> Tip: the first-ever failure of such a test was caused by the
  `Tcl_PkgProvides()` call in the extension's C file using the wrong
  case for the teaish [`-pkgName`](#pkginfo). If your test fails
  without much useful information, check that first.

Also:

> Sidebar: Installing an extension on [Haiku][] requires that one
  manually pass `TCLLIBDIR=/path/to/tcl/lib/tcl<VERSION>` to
  `configure` because the `TCLLIBDIR` is otherwise set to a read-only
  mount point on Haiku. There doesn't seem to be a great way to
  automate that kludge, as we rely entirely on `tclConfig.sh` and
  `tclsh` to tell us where to install the lib, and they provide a
  literally useless (read-only) path on Haiku.


To uninstall:

>
```
$ make uninstall
```

That does nothing more than delete the directory into which the
extension is installed. (_Presumably_ only that extension is installed
in it.)

<a id="files"></a>
Extension Definition Files
========================================================================

This section covers the various files an extension may include to
influence aspects of its build or execution.


<a id="tcl"></a>
`teaish.tcl`
------------------------------------------------------------------------

This script conceptually needs to do several things, though all of
them have usable (though perhaps not useful) defaults.

This script is loaded by teaish very early on in the bootstrapping
process so that the extension may extend the set of `--flags`
available to the configure script. That enables the extension to
influence, for example, the `--help` text.

When the script is initially loaded, it ***must not invoke any
teaish-defined APIs***, as the configuration API is not yet fully
bootstrapped when the extension script is initially read.  Any
exceptions to this rule in the teaish API are clearly noted in their
API docs.

However, the script _is_ expected to run tell teaish a little bit about
the extension by calling `teaish-pkinfo-set` with a set of flags, like:

<a name="pkginfo"></a>

>
```
teaish-pkginfo-set -name foo -version 0.0.1
```

It has a number of flags, all of which have defaults.

- **`-name name`**\  
  The extension's name. It defaults to the name of the directory
  containing the extension. (In TEA this would be the `PACKAGE_NAME`,
  not to be confused with...)
- **`-pkgName pkg-provide-name`**\  
  The extension's name for purposes of `Tcl_PkgProvide()`,
  `package require`, and friends. It defaults to the `-name`, and is normally
  the same, but some projects (like SQLite) have a different name here
  than they do in their historical TEA `PACKAGE_NAME`.
- **`-version the-version`**\  
  The extension's package version. Defaults to 0.0.0.
- **`-libDir dirname`**\  
  The base name of the directory into which this extension should be
  installed. It defaults to a concatenation of `-pkgname` and
  `-version`.
- **`-loadPrefix symbol`**\  
  For use as the second argument passed to [Tcl's `load`
  command][tcl-load] in the package-loading process. It defaults to
  title-cased `-pkgName` because Tcl's `load` plugin system expects it
  in that form.
- **`-vsatisfies {...}`**\  
  Expects a version pattern for use with Tcl's `package vsatisfies`
  command. Teaish ses it in its default [`pkgIndex.tcl.in`](#pkgindex)
  and [test-runner](#test) templates to verify that the package is
  being loaded with an appropriate version of Tcl. The default value
  is "8.5-", as 8.5 is the minimum version teaish will run on, but
  some extensions may require newer versions.
- **`-options {...}`**\  
  If provided, it must be a list compatible with Autosetup's
  `options-add` function. These can also be set up via
  `teaish-options`, described below.

`teaish-pkginfo-set` may be called multiple times to set some options
in later calls, but it must be called (if at all) by the time
`teaish-options` (see below) returns. Calling it from
`teaish-configure` (see below) may confuse the configuration process.

Next, it _may_ define the following functions to influence the
downstream configure process...

- **`teaish-options`**\  
  Gets called very early in the configure process, before the
  validation of the configure-time `--flags`. Must return an empty
  string or a list in the format used by the [Autosetup][]'s `options`
  function. This lets the extension extend the supported `--flags` for
  the configure process. The flags must not collide with those defined
  by Autosetup or teaish - if they do then bootstrapping of teaish
  will fail with an error to the effect that the offending flag has
  already been defined.  Autosetup's built-in `[options-add]` can be
  used to conditionally add new `--flags` from within this function -
  they will all be collapsed into a single list when it's time to
  process the `--flags`.
- **`teaish-configure`**\  
  This gets called after the configuration process is bootstrapped and
  is where the extension may perform arbitrary work to configure the
  build, e.g. by processing `--flags`, performing feature detection,
  etc. It has access to the full Autosetup API, plus the APIs defined
  by [teaish/core.tcl][], [proj.tcl][], and any Autosetup modules it loads
  with the `use` command. After `teaish-configure` completes,
  teaish will wrap up the configure process and write out any pending
  generated files. If `teaish-configure` generates an error,
  configuration will (unsurprisingly) fail.

To reiterate:

> Aside from where specifically noted, most teaish Tcl APIs (named
  `teaish-*`) are not valid to call until `teaish-configure` is
  called, at which point teaish has completed
  bootstrapping. `teaish-options` implementations must avoid the
  temptation to call arbitrary `teaish-*`, deferring that until
  `teaish-configure`.
>
> The only `teaish-*` APIs which are legal to call from the global
  scope, when the script is initially imported, is
  `teaish-pkginfo-set` and `teaish-pragma` (and then only designated
  pragmas are legal that early on).

See [the hello example extension][hello.tcl] for examples.


<a id="make"></a>
`teaish.make.in`
------------------------------------------------------------------------

This _optional_ makefile snippet gets `include`d by the main Makefile
and is intended to communicate info about how to build the
extension.

This file is optional because the same information can be provided
programmatically via `teaish-configure` (from [`teaish.tcl`](#tcl)),
but a makefile may be more convenient for some cases. It's also
possible to programmatically generate arbitrary makefile code and
inject that into the generated makefile instead of, or in addition to,
`teaish.make.in` using `teash-append-make`.

See [](#filtering) for info on how makefiles get filtered.

At its simplest, this file looks something like:

>
```
# List of C source and/or object files:
tx.src = $(tx.dir)/src/tclsqlite3.c

# List of files to include with "make dist". Must be relative to
# the extension's dir.
tx.dist.files += src/tclsqlite3.c

# List of LDFLAGS:
tx.LDFLAGS  = @LDFLAGS_MATH@ @LDFLAGS_ICU@ @LDFLAGS_RT@ @LDFLAGS_ZLIB@

# List of CFLAGS/CPPFLAGS:
tx.CFLAGS   = @CFLAGS_ICU@ @OPT_FEATURE_FLAGS@
```

The main vars are:

- **`tx.dir`** is the absolute path to the directory which holds the
  extension's build files.
- **`tx.src`** should be assigned the source file(s) to compile. It
  can also be assigned object files.  They should generally be
  prefixed with `$(tx.dir)` so that they can be found by out-of-tree
  build. To set these from **`teaish-configure`**, use the
  `teaish-src-add` function.\  
  (C++ files are untested here, but might work if they're _not_ mixed
  together with C files.)
- **`tx.CFLAGS`** should be populated to set any `CFLAGS` or `CPPFLAGS`
  flags which are required for compiling this extension. To set these from
  `teaish-configure`, use the `teaish-cflags-add` function.
- **`tx.LDFLAGS`** should be assigned any linker flags.  To
  set these from `teaish-configure`, use the
  `teaish-add-ldflags` or `teaish-ldflags-prepend` functions.

("tx" is short for "teaish extension.")

The complete list of available variables, and their documentation, are
in the generated `Makefile`, which gets filtered from
[](/file/teaish/Makefile.in).

Note that the top-level makefile _does not_ honor `CFLAGS`, `CPPFLAGS`
and `LDFLAGS` passed directly to `make`. Instead, they must be passed
to `configure`, and they'll get baked into the resulting makefile
under a different name. (This behavior was adopted after seeing it
used to good effect in other trees. It wasn't my idea, but it's a
useful one for configure-scripted projects.)

<a id="hooks"></a>
### Hooking into Various Build Phases


`teaish.make.in` may hook into the `clean`, `distclean`, `test`,
`install`, `uninstall`, and `dist` targets by using make dependencies,
as demonstrated below. Each of those targets is made up of layers of
targets following this pattern:

- **`target`**: does nothing but depends on...
- **`target-post`**: does nothing but depends on...
- **`target-core`**: holds the primary recipe. It depends on...
- **`target-pre`**:
   - **`clean`**, **`distclean`**, **`dist`**, and **`uninstall`**:
     `target-pre` does nothing but clients may make it depend on their
     own targets to run things before `target-core` is run.
   -   **`install`** and **`test`**: depends on **`target-prepre`**
     and does nothing except to act as a hook for for clients.
      - **`install-prepre`** and **`test-prepre`**: both depend on the
        extension DLL. In order for clients to ensure that their `pre`
        targets do not run in parallel with the DLL build when using
        `make -j#`, they should add a `install-pre:mytarget`
        dependency and a `mytarget:$(tx.dll)` dependency (likewise for
        `test-pre`).
- **`target-extension`** is reserved for use by
  [teaish.make](#make).  It has no dependencies and nothing depends
  on it unless the extension defines any such relationships.

Put differently, the dependencies look like:

- `clean`, `distclean`, and `uninstall`: `target` → `target-post` → `target-core` →
  `target-pre`
- `test` and `install`: `target` → `target-post` → `target-core` →
  `target-pre` → `target-prepre`

In the simplest case, they can be hooked into like this:

>
```
clean: clean-extension
clean-extension:
    rm -f my-stuff.foo
```

The pattern is the same for each, but there's a caveat when hooking
into the `test` and `install` targets: one has to be careful to
account for parallel builds.

If running targets in parallel (using `make -j#`) is safe for one's
build then the pattern shown above is sufficient. If, however, their
order needs to be tightly controlled, follow this pattern instead:

>
```
test-post: test-extension
test-extension: test-core
    ...do stuff here...
```

That will ensure that the main tests (`test-core`) run before
`test-extension`, even when building in multi-process mode using
make's `-j#` flag.


<a id="pkginit"></a>
`teaish.pkginit.tcl`
------------------------------------------------------------------------

This optional script is intended to hold code which should be run
immediately after the extension library is loaded.

If `teaish.pkginit.tcl.in` is found in the extension directory, it
is used to generate the `teaish.pkginit.tcl`, [filtering it the
same way makefiles are](#filtering). If not, a static
`teaish.pkginit.tcl` is used if it's available.

- During [`make install`](#install) `teaish.pkginit.tcl` will be
  copied to the extension's installation dir and the generated
  `pkgIndex.tcl` will `source` it after loading the extension.
- During [`make test`](#test) the test wrapper script will `source` it
  after loading the library, before `source`'ing `teaish.test.tcl`
  (if available).

In the context of the `source` calls, a scope-local var named `dir`
will be set which holds the name of the directory from which the file
is loaded. This is derived from conventional `pkgIndex.tcl` usage.


<a id="pkgindex"></a>
`pkgIndex.tcl.in`
------------------------------------------------------------------------

By default a `pkgIndex.tcl` will be generated for an extension based
on [teaish's primary pkgIndex
template](/file/teaish/pkgIndex.tcl.in). A package may override
that in one of two ways:

- Place a `pkgIndex.tcl.in` file in the extension's dir. It is
  [filtered](#filtering) to create `pkgIndex.tcl`.

- Manage a static/hand-written `pkgIndex.tcl` by doing _both_ of the
  following:
  - Place a `pkgIndex.tcl` file in the extension's dir.
  - Add the following line somewhere in the top scope of [teaish.tcl][]:  
  `teaish-pragma --have-own-pkgIndex.tcl`

That combination is required to ensure that teaish will not overwrite
your `pkgIndex.tcl`. The separate step is required because teaish
cannot otherwise distinguish between a hand-written `pkgConfig.tcl`
and one generated in a prior build.


<a id="test"></a>
`teaish.test.tcl`
------------------------------------------------------------------------

The `configure` process generates a file called `teaish.tester.tcl` in
the build directory[^tester-in]. It acts as a wrapper for running arbitrary tests
on the extension.

The extension may optionally include _either_ a `teaish.test.tcl.in`
(subject to [filtering](#filter)) or a `teaish.test.tcl` (not
filtered). In the former case `configure` will generate the latter
from the former.

[^tester-in]: Potential TODO: also support
    `teaish.tester.tcl.in`. This brings with it similar hassles as
    supporting a static `pkgIndex.tcl` (search
    [core.tcl](/file/teaish/autosetup/teaish/core.tcl) for
    `pkgindex-policy` for an example of that rat's tail.

The `make test` target invokes `teaish.tester.tcl` with three arguments:

1. The absolute path of the module's DLL file.

2. The module's "load prefix", for use as the 2nd argument
   [to Tcl's `load` function][tcl-load].

3. The absolute path of [teaish/tester.tcl][], which provides a small
   library of routines which are helpful for running tests.

`teaish.tester.tcl` will do the following before running the tests:

- Use `package vsatisfies` to ensure that the the currently-running
  Tcl version matches the project's [`-vsatisfies` flag](#tcl).
- Load the module.
- `source` [teaish/tester.tcl][].
- If the project includes a [`pkginit` file](#pkginit) then it
  `source`s that file.
- If the project includes a file named [`teaish.test.tcl`](#test) then:
     - _Possibly_ perform some other mysterious bookkeeping or
       bootstrapping.
     - `source` that file.
- If `teaish.test.tcl` is not available then it simply emits a note
  that the module was successfully loaded.

Before `source`'ing the pkginit and test files, it sets a call-local
`dir` var to the name of the directory from which `teaish.pkginit.tcl`
resp.  `teaish.test.tcl` were loaded. (This convention is adopted from
Tcl's `package` command.)

Test script may run arbitrary Tcl code for testing. If the script
propagates an error, `make test` will fail.  [teaish/tester.tcl][]
(see above) provides a set of helpful routines for running basic
tests.

Here are some [example tests from the SQLite module](/file/teaish/example/sqlite/teaish.test.tcl):

>
```
test-expect 1.0-open {
  sqlite3 db :memory:
} {}

test-expect 1.1-select {
  db eval {select 'hi, world',1,2,3}
} {{hi, world} 1 2 3}

test-expect 99.0-db-close {db close} {}
```

Tests which need to read or write files need to take care to do so
from/to the proper directory so that they will work properly when
invoked via out-of-tree builds:

- The test script itself is available as `$::argv0` and its containing
  directory can be derived from that using `[file dirname $::argv0]`.
- Tcl's builtin `pwd` command will give the directory from which the
  test script was invoked, which will differ from the script's
  directory when doing an out-of-tree build.


<a id="dist"></a>
Distributing Extensions
========================================================================

Teaish provides basic support for generating a "dist" zip file and
tarball for extensions by simply running `make dist`. The distribution
includes _only_ the extension, not the whole teaish
bundle[^todo-megadist].

[^todo-megadist]: Though a potential TODO would be to pack both
  together, simply dropping the extension into the main teaish dir and
  packaging them up together.

These `dist` rules can be disabled by calling `teaish-enable-dist 0`
from [`teaish-configure`](#tcl), in which case `dist`
is left undefined so that the client makefile may define it. When,
however, doing an out-of-tree build, the `dist` rules are
_unconditionally disabled_ because handling of the paths for archival
purposes becomes troublesome to do from Makefile code.

By default, teaish will automatically include the following files into
the distribution bundle if they are present:

- [`teaish.tcl`](#tcl)
- [`teaish.test.tcl`](#test)
- [`teaish.pkginit.tcl`](#pkginit)
- [`teaish.pkgIndex.tcl.in`](#pkgindex) or (in one case)
  [`teaish.pkgIndex.tcl`](#pkgindex)
- [`teaish.make.in`](#make)

In addition, it will include any files or directories which are either:

- Passed to `[teaish-dist-add]` or `[teaish-src-add -dist]` via
  `[teaish-configure]` (in `teaish.tcl`).
- _Appended_ to the makefile var `$(tx.dist.files)` via
  `[teaish.make.in]`. If `teaish.make.in` overwrites that that
  variable then the automatically-added files will _not_ be included.

Caveat(s):

- "Dist" file names _must_ be relative to the extension dir.

Running `make dist` will create:

- `$TEAISH_NAME-$TEAISH_VERSION.zip` if the `zip` binary is found.
- `$TEAISH_NAME-$TEAISH_VERSION.tar.gz` if the `tar` binary is
  found (noting that it assumes a `tar` which supports `czf`, which
  some older `tar` implementations do not).

Each archive has a top-level directory named
`$TEAISH_NAME-$TEAISH_VERSION`.

The `clean` and `distclean` targets _do not_ delete those, but
the `undist` target will.

To build a freshly-unpackaged extension, do:

>
```
$ /path/to/teaish/configure ...
$ make
```

<a id="filtering"></a>
Template File Filtering
========================================================================

Several of the files mentioned in this doc are filtered in a manner
conventional to GNU Autotools. Specifically, the follow the rules for
Autosetup's `make-template` function, but the conventions are similar.
In this project, all such files have a suffix of `.in`.

Input files may have any number of `@PLACEHOLDERS@` in them, which
will be filtered out during the configure process, each one mapped to
an Autosetup `define`, and the results will be written to some target
file (typically the same as the input's name minus any `".in"`
extension). *All such placeholders must be resolved*, using values
provided by Autosetup, teaish, or set in `teaish.tcl` with Autosetup's
`define` function. If any unresolved placeholders remain after the
filtering step, `configure` will fail[^validation-exception].

Filering of such files happens very late in the `configure` process,
as it must happen after all `define`s are set up.


[^validation-exception]: Exception: commented-out lines in files named
  `*[Mm]ake*` are ignored _if_ the line starts with `#` (ignoring
  leading whitespace). Aside from that exception, the content is
  semantically opaque to the validation, so it has no notion of what a
  "comment" is.


[Autosetup]: https://msteveb.github.io/autosetup/
[tcl-load]: https://www.tcl-lang.org/man/tcl8.6/TclCmd/load.htm
[proj.tcl]: /file/teaish/autosetup/proj.tcl
[teaish/core.tcl]: /file/teaish/autosetup/teaish/core.tcl
[teaish/tester.tcl]: /file/teaish/autosetup/teaish/tester.tcl
[hello.tcl]: /file/teaish/example/hello/teaish.tcl
[Haiku]: https://www.haiku-os.org/
[teaish.tcl]: #tcl
